import fs from "fs";
import forge from "node-forge";
import crypto from "crypto";
class ConfigCfdi {
    constructor(options) {
        this.cert = {
            noCertificado: "",
            pem: "",
            validity: { notAfter: "", notBefore: "" },
            version: 0,
        };
        this.key_pem = "";
        this.password = options.password;
        this.readCert(options.cert_path);
        this.readKey(options.key_path);
    }
    readCert(path) {
        try {
            const file = fs.readFileSync(path);
            const certAsn1 = forge.asn1.fromDer(file.toString("binary"));
            const cert = forge.pki.certificateFromAsn1(certAsn1);
            if (!cert) {
                throw new Error("Certificate not found or invalid format");
            }
            this.cert = {
                noCertificado: cert.serialNumber
                    .match(/.{1,2}/g)
                    .map(function (v) {
                    return String.fromCharCode(parseInt(v, 16));
                })
                    .join(""),
                pem: forge.pki.certificateToPem(cert),
                validity: {
                    notAfter: cert.validity.notAfter.toString(),
                    notBefore: cert.validity.notBefore.toString(),
                },
                version: cert.version,
            };
            const issuer = cert.issuer.attributes.map((attr) => `${attr.shortName}=${attr.value}`).join(", ");
            console.log(cert.issuer);
        }
        catch (error) {
            throw new Error(`Error reading certificate: ${error}`);
        }
    }
    readKey(path) {
        try {
            const file = fs.readFileSync(path);
            const pem = crypto.createPrivateKey({
                key: file,
                format: "der",
                type: "pkcs8",
                passphrase: this.password,
            });
            const pemString = pem.export({ format: "pem", type: "pkcs8" });
            this.key_pem = pemString;
        }
        catch (error) {
            throw new Error(`Error reading key: ${error}`);
        }
    }
    getCert() {
        return this.cert;
    }
    getKeyPem() {
        return this.key_pem;
    }
}
export default ConfigCfdi;
