export interface INodeComprobante {
    serie: string;
    folio: string;
    fecha: string;
    subtotal: string;
    formaPago: string;
    total: string;
    metodoPago: "PUE" | "PPD";
    lugarExpedicion: string;
    tipoDeComprobante?: "I" | "E" | "T";
    moneda?: string;
    exportacion?: string;
    condicionesDePago?: string;
    descuento?: string;
    tipoCambio?: string;
}
export interface IObjectNodeComprobante {
    "xsi:schemaLocation": string;
    "xmlns:cfdi": string;
    "xmlns:xsi": string;
    Version: string;
    Serie: string;
    Folio: string;
    Fecha: string;
    SubTotal: string;
    Moneda: string;
    FormaPago: string;
    Total: string;
    MetodoPago: "PUE" | "PPD";
    TipoDeComprobante: "I" | "E" | "T";
    LugarExpedicion: string;
    NoCertificado: string;
    Certificado: string;
    Exportacion: string;
    CondicionesDePago?: string;
    Descuento?: string;
    TipoCambio?: string;
}
export interface INodeInformacionGlobal {
    exist: boolean;
    periodicidad: string;
    meses: string;
    anio: string;
}
export interface INodeRelacionados {
    tipoRelacion: string;
    uuids: string[];
}
export interface INodeEmisor {
    rfc: string;
    nombre: string;
    regimenFiscal: string;
    facAtrAdquirente?: string;
}
export interface IObjectNodeEmisor {
    Rfc: string;
    Nombre: string;
    RegimenFiscal: string;
    FacAtrAdquirente?: string;
}
export interface INodeReceptor {
    rfc: string;
    nombre: string;
    domicilioFiscal: string;
    regimenFiscal: string;
    usoCfdi: string;
    residenciaFiscal?: string;
    numRegIdTrib?: string;
}
export interface IObjectNodeReceptor {
    Rfc: string;
    Nombre: string;
    DomicilioFiscalReceptor: string;
    RegimenFiscalReceptor: string;
    UsoCFDI: string;
    ResidenciaFiscal?: string;
    NumRegIdTrib?: string;
}
export interface INodeConcepto {
    concepto: INodeConc;
    impuestos?: INodeImp;
    aCuentaTerceros?: INodeACuentaTerceros;
    informacionAduanera?: INodeInformacionAduanera;
    cuentaPredial?: {
        numero: string;
    };
    complementoConcepto?: INodeComplementoConcepto[];
    parte?: INodeParte[];
}
export interface INodeConc {
    claveProdServ: string;
    cantidad: string;
    claveUnidad: string;
    descripcion: string;
    valorUnitario: string;
    importe: string;
    unidad: string;
    objetoImp?: "01" | "02" | "03";
    noIdentificacion?: string;
    descuento?: string;
}
export interface IObjectNodeConcepto {
    ClaveProdServ: string;
    Cantidad: string;
    ClaveUnidad: string;
    Descripcion: string;
    ValorUnitario: string;
    Importe: string;
    ObjetoImp: "01" | "02" | "03";
    Unidad?: string;
    NoIdentificacion?: string;
    Descuento?: string;
}
interface INodeImp {
    traslados?: INodeTraslado[];
    retenciones?: INodeRetencion[];
}
export interface INodeTraslado {
    base: string;
    impuesto: string;
    tipoFactor: string;
    tasaOCuota?: string;
    importe: string;
}
export interface IObjectNodeTraslado {
    Base: string;
    Impuesto: string;
    TipoFactor: string;
    TasaOCuota?: string;
    Importe: string;
}
export interface INodeRetencion {
    base: string;
    impuesto: string;
    tipoFactor: string;
    tasaOCuota: string;
    importe: string;
}
export interface IObjectNodeRetencion {
    Base: string;
    Impuesto: string;
    TipoFactor: string;
    TasaOCuota: string;
    Importe: string;
}
export interface INodeACuentaTerceros {
    rfcACuentaTerceros: string;
    nombreACuentaTerceros: string;
    regimenFiscalACuentaTerceros: string;
    domicilioFiscalACuentaTerceros: string;
}
export interface IObjectNodeACuentaTerceros {
    RfcACuentaTerceros: string;
    NombreACuentaTerceros: string;
    RegimenFiscalACuentaTerceros: string;
    DomicilioFiscalACuentaTerceros: string;
}
interface INodeInformacionAduanera {
    numeroPedimento: string;
}
export interface INodeComplementoConcepto {
    nodo: string;
    atributos: Record<string, string>;
}
export interface INodeParte {
    concepto: INodeConcParte;
    informacionAduanera?: INodeInformacionAduanera;
}
export interface INodeConcParte extends Omit<INodeConc, "claveUnidad" | "objetoImp" | "descuento" | "valorUnitario" | "importe"> {
    valorUnitario?: string;
    importe?: string;
}
export interface IObjectNodeParte extends Omit<IObjectNodeConcepto, "ClaveUnidad" | "ObjetoImp" | "Descuento" | "ValorUnitario" | "Importe"> {
    ValorUnitario?: string;
    Importe?: string;
}
export {};
